/*
 * libCcsMox_v6.h
 *
 *  Author: Richard Lai
 *  Copyright: Copyright Cambridge Cmos Sensors 2014
  *
 *   Updated: 2014-09-25
 */ 


#ifndef INCLIBCCSMOXV6_H_
#define INCLIBCCSMOXV6_H_


uint8_t CCS_PROFILE_ACO[] = {
        0x19, 0xe0, 0x48, 0x48, 0x1, 0x3a, 0xf, 0x69, 0x49, 0x4, 0x1d, 0x71, 0x66, 0x1f, 0x80, 0x95,
        0xbf, 0xd4, 0x15, 0x34, 0xb2, 0x28, 0x56, 0x6c, 0x76, 0x8b, 0x7f, 0x66, 0xaf, 0x6a, 0x38, 0x12
};

uint8_t CCS_PROFILE_VOC[] = {
        0x32, 0x55, 0xfb, 0xe8, 0xa6, 0xd2, 0x6e, 0x81, 0x63, 0x1c, 0xe2, 0x55, 0x45, 0x12, 0xf4, 0xf8,
        0x1e, 0xf1, 0x5c, 0x47, 0xc2, 0xa8, 0x15, 0x81, 0xc7, 0xfa, 0xca, 0x2c, 0x50, 0x20, 0x38, 0xea
};

uint16_t CCS_ALARM_LEVELS[] = {
        5000,7000,8500,9500,10000
};

extern uint8_t ccsmox_set_profile(uintptr_t ccs_profile, uint8_t size);
extern uint8_t ccsmox_set_conf(uint8_t conf, uintptr_t ccs_data, uint8_t size);
extern uint8_t ccsmox_get_conf(uint8_t conf, uintptr_t ccs_data, uint8_t size);
extern uint8_t ccsmox_set_data(uintptr_t ccs_data, uint8_t size);
extern uint16_t ccsmox_set_data_iaq(uintptr_t ccs_data, uint8_t size);

typedef struct CcsValueRef
{
	uint32_t value;
    uint32_t timestamp;
} CcsValueRef;

typedef struct CcsValueAttribute
{
    uint8_t voltage;
    uint8_t temperature;
    uint8_t humidity;
    uint8_t pressure;
} CcsValueAttribute;

typedef struct CcsValue
{
    CcsValueRef ref;
    CcsValueAttribute attributes;
    uint8_t level;
    uint8_t percentage;
    uint16_t concentration;
} CcsValue;

typedef struct CcsPulseConf
{
    uint8_t voltage;
    uint16_t duration;
    uint16_t interval;
} CcsPulseConf;

typedef struct CcsConf
{
    uint8_t type;
    uint8_t size;
    uintptr_t buffer;
} CcsConf;

#define DEFAULT_CONF_PULSE_SIZE 64
#define DEFAULT_CONF_BUFFER_SIZE 64

#define CONF_RESET 0x00
#define CONF_GET_POWER 0x02
#define CONF_GET_CAL 0x03
#define CONF_SET_PROFILE 0x81
#define CONF_SET_POWER 0x82
#define CONF_SET_CAL 0x83
#define CONF_SET_ALARM 0x84

#define TYPE_ALCOHOL 0x00
#define TYPE_CO 0x01
#define TYPE_VOC 0x02

#define IND_GAS_LOW 0x00
#define IND_GAS_MEDIUM 0x01
#define IND_GAS_HIGH 0x02
#define IND_CAL_OK 0x41

#define ERROR_OK 0
#define ERROR_NOT_READY 1
#define ERROR_INVALID_BUFFER 0x21
#define ERROR_INSUFFICIENT_BUFFER 0x31
#define ERROR_INVAILD_PARAMETER 0x22

#endif /* INCLIBCCSMOXV6_H_ */
