/**********************************************************************
 *HY16F188
 * main.c
 * Created on:2013/12/08
 * Maintenance:2014/10/01
 * --------------------------------------------------------------------
 * Release 1.0.0
 * --------------------
  * HY16F184(I2C Slave)
 * ---------			    ---------------
 *          |              |                |
 *    PT2.7 | SDA <--> SDA |   			    |
 *    PT2.6 | SCK <--- SCK |        		|
 *    PT1.7 | PWM --->     |                |
 *     GND  |              |                |
 *          |              |                |
 **********************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvI2C.h"
#include "DrvCLOCK.h"
#include "my define.h"
#include "ccs_types.h"
#include "libCcsMox_v6.h"
#include "DrvADC.h"
#include "DrvTimer.h"
#include "stdlib.h"
#include "DrvPMU.h"
#include "DrvOP.h"
#include "DrvREG32.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_I2C_TxDone:1;
    unsigned b_I2C_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define PWM_ADC_StartTime 207 //50ms Start measuring ADC
#define PWM_ADC_StartTime 290 //70ms Start measuring ADC
//#define PWM_ADC_StartTime 373 //90ms Start measuring ADC
#define PWM_Heater_FinishTime 414 //100ms
//#define PWM_Heater_FinishTime 621 //150ms
//#define PWM_Heater_FinishTime 828 //200ms
#define PWM_PeriodTime 2074 //500ms
#define I2CBufferSize 256

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;

unsigned int TMA_Count;
unsigned char ADfinish;
unsigned char ADC_Count;
unsigned char I2C_Done;
char RS_LowByte, RH_LowByte, PPM_LowByte;
char RS_MiddleByte, RH_MiddleByte, PPM_MiddleByte;
char RS_HighByte, RH_HighByte, PPM_HighByte;
int RS,RH_Current;
int PPM;
int ADCData;
int i;

// Constants for I2C
unsigned char I2C_TARGET;                  // Target I2C slave address
unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
unsigned int DataTxLen,DataTxIndex,DataRxLen,DataRxIndex,EndFlag;
unsigned int GeneralFlag;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void ADchange(void);
void InitalADC(void);
void HWI2C_INI(void);
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{

  struct CcsValue value = {{0,0},{0,0,0,0},0,0,0};  //Initial all data =0
  for(i=0;i<=256;i++)  //if 6=HY14E10 Test tool AP
  {
    I2C_Sendbuf[i]=0;  //Initial I2C data buffer=0
	I2C_Recbuf[i]=0; //Initial I2C data buffer=0
  }

  ADC_Count=0;
  TMA_Count=0;
  I2C_Done=0;
  ADfinish=0;
  RS_LowByte=0, RH_LowByte=0, PPM_LowByte=0;
  RS_MiddleByte=0, RH_MiddleByte=0, PPM_MiddleByte=0;
  RS_HighByte=0, RH_HighByte=0, PPM_HighByte=0;

  HWI2C_INI();
  InitalADC();
  SYS_EnableGIE(7,0x3F);  // Enable GIE(Global Interrupt)

  DrvGPIO_Open(E_PT1,0x80,E_IO_OUTPUT);  //PT1.7 set Output, PWM1 Output pin
  //DrvGPIO_Open(E_PT1,0x40,E_IO_OUTPUT);  //PT1.6 set Output, check ADC measure time
  DrvTMBC_Clk_Source(0,0);  //TimerB Clock enable HS_CK, PerScale=1
  DrvTMB_Open(E_TMB_MODE0,E_TMB_NORMAL,0x00c8);  //TimerB overflow 0Xc8->PWM period 0xc8
  DrvPWM1_Open(0,1,3);  //PWM1 Enable, Port 1.7=PWMO1(Output PWM signal, PWM frequency=97us, Low=57us, High=40us
  DrvPWM_CountCondition(0xffff,0x0078);			    //To control PMOS, PWM1 Duty 0x78, PWM0 Duty 0xffff(PWM0 No use)
  DrvTMA_Open(3,0);                    //TimerA Overflow, TMA Speed, 250us
                                       //3:taclk/16, if ENTAD=1, taclk=32
                                       //00:HS_CK
  DrvTIMER_ClearIntFlag(E_TMA);        //Clear Timer A interrupt flag
  DrvTIMER_EnableInt(E_TMA);           //Timer A interrupt enable

  //Following to do Volatile Organic Compounds VOC calculation
  struct CcsConf conf = { TYPE_VOC, sizeof(CCS_PROFILE_VOC), (uintptr_t)&CCS_PROFILE_VOC};
  ccsmox_set_conf(CONF_SET_PROFILE, (uintptr_t)&conf, sizeof(conf));

  while(1)
  {
    if(ADfinish==1)
	{
	  ADfinish=0;
	  RH_Current=RH_Current>>(3);	//RH4chopper, ҥHn/8ʧ@.
	  value.ref.value = RS;
	  ccsmox_set_data_iaq( (uintptr_t)&value, sizeof(value) );
	  PPM=value.concentration & 0xffff;

	  PPM_LowByte=(PPM << 1) & 0xFF;
	  PPM_MiddleByte=(PPM >> 7) & 0xFF;
	  PPM_HighByte=(PPM >> 15) & 0xFF;
	  RS_LowByte=(RS >> 0) & 0xFF;
	  RS_MiddleByte=(RS >> 8) & 0xFF;
	  RS_HighByte=(RS >> 16) & 0xFF;
	  RH_LowByte=(RH_Current >> 0) & 0xFF;
	  RH_MiddleByte=(RH_Current >> 8) & 0xFF;
	  RH_HighByte=(RH_Current >> 16) & 0xFF;

	  I2C_Sendbuf[13]=PPM_LowByte;
	  I2C_Sendbuf[14]=PPM_MiddleByte;
	  I2C_Sendbuf[15]=PPM_HighByte;
	  I2C_Sendbuf[16]=RS_LowByte;
	  I2C_Sendbuf[17]=RS_MiddleByte;
	  I2C_Sendbuf[18]=RS_HighByte;
	  I2C_Sendbuf[19]=RH_LowByte;
	  I2C_Sendbuf[20]=RH_MiddleByte;
	  I2C_Sendbuf[21]=RH_HighByte;
	  I2C_Sendbuf[22]=0xff;
	  I2C_Sendbuf[23]=0xff;
	  I2C_Sendbuf[24]=0xff;
	  I2C_Done=1;  //Enable I2C Slave data Buffer Update
	}

  }
  return 0;

}

/*---------------------------------------------------------------------------*/
/* Hardware Communication Interrupt                                          */
/* UART/SPI/I2C Interrupt Service Routines                                   */
/*---------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status;

  if(DrvI2C_ReadIntFlag()==E_DRVI2C_INT)  // Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();  // Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x44: //SACTFlag+ACKFlag
          { //Own slave A + W has been received. ACK has been transmitted.
            DataRxIndex=0;
            MCUSTATUSbits.b_I2C_RxDone=1;
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x4C: //SACTFlag+DFFlag+ACKFlag
          { //Data byte has been received. ACK has been transmitted.
            I2C_Recbuf[DataRxIndex]=DrvI2C_ReadData();
            if(I2C_Recbuf[DataRxIndex]==0x80)
            {
                if(I2C_Done==1)
                {
        	        I2C_Sendbuf[1]=I2C_Sendbuf[13]|0x1; //bit0=1b, new data
        	    	I2C_Sendbuf[2]=I2C_Sendbuf[14];
        	    	I2C_Sendbuf[3]=I2C_Sendbuf[15];
        	    	I2C_Sendbuf[4]=I2C_Sendbuf[16]|0x1;  //bit0=1b, new data
        	    	I2C_Sendbuf[5]=I2C_Sendbuf[17];
        	    	I2C_Sendbuf[6]=I2C_Sendbuf[18];
        	    	I2C_Sendbuf[7]=I2C_Sendbuf[19]|0x1;  //bit0=1b, new data
        	    	I2C_Sendbuf[8]=I2C_Sendbuf[20];
        	    	I2C_Sendbuf[9]=I2C_Sendbuf[21];
        	    	I2C_Sendbuf[10]=I2C_Sendbuf[22]|0x1;  //bit0=1b, new data
        	    	I2C_Sendbuf[11]=I2C_Sendbuf[23];
        	    	I2C_Sendbuf[12]=I2C_Sendbuf[24];
        	    	I2C_Done=0;
                }
                else
                {
                	I2C_Sendbuf[1]=(I2C_Sendbuf[1]&0xFE);  //bit0=0b, old data
        	    	I2C_Sendbuf[2]=I2C_Sendbuf[2];
        	    	I2C_Sendbuf[3]=I2C_Sendbuf[3];
        	    	I2C_Sendbuf[4]=(I2C_Sendbuf[4]&0xFE);  //bit0=0b, old data
        	    	I2C_Sendbuf[5]=I2C_Sendbuf[5];
        	    	I2C_Sendbuf[6]=I2C_Sendbuf[6];
        	    	I2C_Sendbuf[7]=(I2C_Sendbuf[7]&0xFE);  //bit0=0b, old data
        	    	I2C_Sendbuf[8]=I2C_Sendbuf[8];
        	    	I2C_Sendbuf[9]=I2C_Sendbuf[9];
        	    	I2C_Sendbuf[10]=(I2C_Sendbuf[10]&0xFE);  //bit0=0b, old data
        	    	I2C_Sendbuf[11]=I2C_Sendbuf[11];
        	    	I2C_Sendbuf[12]=I2C_Sendbuf[12];

                }
            }
            DataRxIndex++;
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x48: //SACTFlag+DFFlag
          { //Data byte has been received. NACK has been transmitted.
            I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          }

      case 0x4A: // I2C General Call
          { //One data byte has been received. NACK has been transmitted
        	GeneralFlag=1;
        	DataRxIndex=0;
        	MCUSTATUSbits.b_I2C_RxDone=1;
        	I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          }
      case 0x4E: // I2C General Call
           { //One data byte has been received. ACK has been transmitted
        	 GeneralFlag=1;
        	 I2C_Recbuf[DataRxIndex++]=DrvI2C_ReadData();
             DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
             break;
           }

      case 0x54: //SACTFlag+RWFlag+ACKFlag
          { //Own slave A + R has been received. ACK has been transmitted.
            MCUSTATUSbits.b_I2C_TxDone=1;
           	DataTxIndex=0;
            DataTxIndex++;  //Robert Add, Read data index
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x55: //SACTFlag+RWFlag
          { //Own slave A + R has been received. NACK has been transmitted.
            DataTxIndex=0;
            MCUSTATUSbits.b_I2C_TxDone=1;
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x5C:
          { //Data byte has been transmitted. ACK has been transmitted.
            DrvI2C_WriteData(I2C_Sendbuf[DataTxIndex++]);  // Send Data to Master
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x58:
          { //Data byte has been transmitted. NACK has been received.
            DataTxLen=DataTxIndex;
            EndFlag=1;
            DrvI2C_WriteData(0x80); // set MSB==High, NACK then STOP
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x30:
          { //A STOP  has been received.
            DataRxLen=DataRxIndex;
            DataTxLen=DataTxIndex;
        	EndFlag=1;
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      default:
          {
            DataTxIndex=0;
            DataRxIndex=0;
            EndFlag=1;
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
    }
    DrvI2C_ClearEIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CIF)
  }
  if(DrvI2C_ReadIntFlag()==E_DRVI2C_ERROR_INT)  //Get I2C Error Interrupt Flag
  {
    EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,1,0,0);   // I2C as master sends STOP signal
  }
}
/*---------------------------------------------------------------------------*/
/* WDT & RTC & Timer A/B/C Interrupt Service Routines                        */
/*---------------------------------------------------------------------------*/
void HW1_ISR(void)
{
  DrvTIMER_ClearIntFlag(E_TMA);		//Clear TMA interrupt flag
  TMA_Count++;

  if(TMA_Count==PWM_ADC_StartTime)  //If TMA count to PWM_ADC_StartTime, Start Measuring ADC
  {
	//DrvGPIO_SetPortBits(E_PT1,0x40);  //check ADC start Time, PT1.6=high
	//DrvGPIO_ClrPortBits(E_PT1,0x40);  //Check ADC start Time, PT1.6=low
    DrvADC_Enable();
	DrvADC_CombFilter(ENABLE);
  }


  if(TMA_Count==PWM_Heater_FinishTime)	//If TMA count to PWM_Heater_FinishTime, PT1.7 set High, PMOS off
  {
    DrvPWM1_Close();
 	DrvGPIO_SetPortBits(E_PT1,0x80);  //PT1.7 set High, PMOS Off
  }

  if(TMA_Count>=PWM_PeriodTime)  //If TMA count to PWM_PeriodTime, repeat to do PWM output
  {
    TMA_Count=0;	// Let TMA count=0=0ms,
    DrvPWM1_Open(0,1,3);  //(PWM output signal, PWM frequency=97us, Low=57us, High=40us
    					  //PWM Output signal from the time ( 0ms ~ PWM_Heater_FinishTime )
  }

}

/*----------------------------------------------------------------------------*/
/* HW2 ADC Interrupt Subroutines                                              */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

  if(DrvADC_ReadIntFlag()) //ŪADC_nDлx줸
  {
    DrvADC_ClearIntFlag();
    ADCData=DrvADC_GetConversionData()>>8;  //get +/-23bit ADO
	ADchange();  //get ADC data, RS and RH data
	ADC_Count++;
	if(ADC_Count>=9)
	{
	  //If DrvADC_OSR(5), spend 44ms, (6):24ms, (7):12ms, (8):7.5ms
	  //DrvGPIO_SetPortBits(E_PT1,0x40);  //check ADC Finish Time, PT1.6=high
	  //DrvGPIO_ClrPortBits(E_PT1,0x40);  //check ADC Finish Time, PT1.6=low
	  ADC_Count=0;
	 }
  }

}

/*---------------------------------------------------------------------------*/
/* CMP/OPA Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*---------------------------------------------------------------------------*/
/* PT1 Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}

/*---------------------------------------------------------------------------*/
/* PT2 Interrupt Service Routines                                            */
/*---------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}

/*---------------------------------------------------------------------------*/
/* Get ADchange Data                                                         */
/*---------------------------------------------------------------------------*/
void ADchange(void)
{
	switch(ADC_Count)
	{
	  case 0:
	  {
		RS=ADCData;
		DrvADC_CombFilter(DISABLE);
		DrvADC_SetADCInputChannel(ADC_Input_AIO0,ADC_Input_AIO3);
		DrvADC_Gain(ADC_PGA_Disable,ADC_Gain_32);
		DrvADC_CombFilter(ENABLE);
		RH_Current=0;
		break;
	  }


	  case 1:
	  {
		RH_Current=ADCData;
		break;
	  }


	  case 2:
	  {
		RH_Current+=ADCData;
		break;
	  }


	  case 3:
	  {
		RH_Current+=ADCData;
		break;
	  }


	  case 4:
	  {
		RH_Current+=ADCData;
		DrvADC_CombFilter(DISABLE);
		DrvADC_SetADCInputChannel(ADC_Input_AIO3,ADC_Input_AIO0);
		DrvADC_CombFilter(ENABLE);
		break;
	  }


	  case 5:
	  {
		RH_Current-=ADCData;
		break;
	  }

		
	  case 6:
	  {
		RH_Current-=ADCData;
		break;
	  }


	  case 7:
	  {
		RH_Current-=ADCData;
		break;
	  }


	  case 8:
	  {
		RH_Current-=ADCData;
		DrvADC_Disable();
		DrvADC_CombFilter(DISABLE);
		DrvADC_SetADCInputChannel(OPO,ADC_Input_AIO1);
		DrvADC_Gain(ADC_PGA_Disable,ADC_PGA_Disable);
		ADfinish=1;
		break;
      }

	  default:break;

	}
}

/*--------------------------------------------------------------------*/
/* Hardware I2C Initial                                               */
/*--------------------------------------------------------------------*/
void HWI2C_INI(void)
{

  DrvI2C_SetIOPin(7);  //Set I2C port, PT2.6=SCL, PT2.7=SDA
  clk_00=0xff01;  //ENHAO=1

  i2c_00=0xff00ff00;//0x41000//i2c off
  i2c_14=0x00ffffff;//0x44014//Transmitter, Data Buffer=1

  i2c_08=0xff200000; //0x41008, Set I2C CRG
  i2c_00=0xff00ff01;//0x41000//ok i2c enable
  i2c_08=0xff04ff7f; //0x41008//I2C Clock Control//Time-out Control Register

  i2c_0c=0x00ff0020;//I2C slaver id_0=0x20
  i2c_04=0xffff80;//0x41004//i2c slaver en
  i2c_0c=0x00000021;//slaver id_0
  asm("NOP");
  int_00=0xff30ff00;  //I2CIE=1, I2CEIE

}

/*--------------------------------------------------------------------*/
/* Hardware ADC Initial                                               */
/*--------------------------------------------------------------------*/
void InitalADC(void)
{

  //Set ADC input pin
  DrvADC_SetADCInputChannel(OPO,ADC_Input_AIO1); //Set the ADC positive=PT3.7/negative=AIO1 input voltage source.
  DrvADC_InputSwitch(OPEN);                //ADC signal input (positive and negative) short(VISHR) control.
  DrvADC_RefInputShort(OPEN);              //Set the ADC reference input (positive and negative) short(VRSHR) control.

  DrvADC_Gain(ADC_PGA_Disable,ADC_PGA_Disable); //Input signal gain for modulator, Set the ADC Gain 1, PGA=1
  DrvADC_DCoffset(0);                      //DC offset input voltage selection (VREF=REFP-REFN)
  DrvADC_RefVoltage(VDDA,0);  			   //Set the ADC reference voltage. (VDDA-VSSA)
  DrvADC_FullRefRange(1);                  //Set the ADC full reference range select.
                                           //0: Full reference range input
                                           //1: 1/2 reference range input
  //For CCS801 CMOS Sensor, ADC acquisition rate at least 1kHz
  DrvADC_OSR(7);                //7 : 256==1280sps
  //DrvADC_OSR(8);             //8:OSR=128, Speed=2.56k sps
  DrvADC_ClkEnable(0,1);       //Setting ADC CLOCK ADCK=HS_CK/6 & Rising edge is high
  //DrvADC_FastChopper(1);     //fast chopper enable

  //Set VDDA voltage
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_BandgapEnable();
  //DrvPMU_REFO_Enable();
  DrvPMU_REFO_Disable();
  DrvPMU_AnalogGround(ENABLE);  //ADC analog ground source selection.
                                //1 : Enable buffer and use internal source(need to work with ADC)
  Delay(1000);

  //Set ADC interrupt
  DrvADC_EnableInt();
  DrvADC_ClearIntFlag();
  //DrvADC_Enable();
  DrvADC_CombFilter(ENABLE);   //CombFilter open

  //Set OPA input pin
  DrvOP_Open();
  DrvOP_PInput(0x01);  //Selection the Rail-to-rail OPAMP positive input of AIO2
  DrvOP_NInput(0x08);  //Selection the Rail-to-rail OPAMP negative input of OPOI
  //DrvOP_OPOoutEnable();

}

/*---------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                */
/*---------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/
